#include <xc.h>
#include <pic16f886.h>
#include "port_init.h"
#include "user_define.h"
#include "wait_time.h"

#pragma config BOREN = OFF
#pragma config CPD = OFF 
#pragma config FOSC =INTRC_NOCLKOUT  
#pragma config MCLRE = ON 
#pragma config WDTE = OFF 
#pragma config CP = OFF 
#pragma config LVP = OFF 
#pragma config PWRTE = ON
#pragma config IESO =OFF
#pragma config FCMEN = OFF

#pragma config BOR4V = BOR21V
#pragma config WRT = OFF

unsigned char	sw1_key;
unsigned char	sw2_key;

unsigned int    enc_count;
unsigned char   rb2_flg;
unsigned char   rb1_flg;
unsigned char   rx_count;
unsigned int    rx_data;
unsigned char   rx_flg;

unsigned int    over_count;
unsigned char   led_font[10]={0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x27,0x7f,0x6f};

unsigned long   disp_time;

// RB interrupt
void interrupt InterTimer( void )
{
    if (RBIF == 1) {
        RC5=1;NOP();

        rb2_flg=RX_CLK;NOP();
        rb1_flg=RX_DATA;NOP();
        if(rb2_flg==1){
            over_count=1;
            rx_data=rx_data | rb1_flg;
            rx_count++;
            if(rx_count>=16){
                rx_flg=1;
            }
            else{
                rx_data=rx_data << 1;
            }
        }
        
        RBIF=0;
        RC5=0;NOP();
    }
}

// RB int init
void    rb_int_init(void)
{
    IOCB=0b00000100;
    RBIF=0;
    RBIE=1;
    PEIE=1;
    GIE=1;
}

// LED Disp
void    com_clear(void)
{
    COM0=1;NOP();
    COM1=1;NOP();
    COM2=1;NOP();
    COM3=1;NOP();
}

void    led_disp(void)
{
unsigned char   n0,n1,n2,n3;
unsigned int    count;

    count=enc_count;
    n0=(unsigned char)(count%10);count=(count-n0)/10;
    n1=(unsigned char)(count%10);count=(count-n1)/10;
    n2=(unsigned char)(count%10);count=(count-n2)/10;
    n3=(unsigned char)(count%10);

    com_clear();
    
    PORTA=~led_font[n0];
    COM0=0;NOP();
    __delay_ms(LED_TIME);
    COM0=1;NOP();

    PORTA=~led_font[n1];
    COM1=0;NOP();
    __delay_ms(LED_TIME);
    COM1=1;NOP();

    PORTA=~led_font[n2];
    COM2=0;NOP();
    __delay_ms(LED_TIME);
    COM2=1;NOP();

    PORTA=~led_font[n3];
    COM3=0;NOP();
    __delay_ms(LED_TIME);
    COM3=1;NOP();
    
    com_clear();
}

// MAIN
void	main(void)
{
    __delay_ms(1000);
	port_init();
    
    rb_int_init();
    
    enc_count=0;
    rx_count=0;
    rx_data=0;
    rx_flg=0;
    
    over_count=0;
    com_clear();
    led_disp();
    disp_time=DISP_TIME_INIT;
    
	while(1){
        
        // RX
        if(rx_flg==1){
            rx_flg=0;
            enc_count=(~rx_data) & 0x03ff;
            rx_count=0;
            rx_data=0;
            disp_time=DISP_TIME_INIT;
        }
        
        // LED Disp
        if(disp_time>0){
            led_disp();            
        }
        
        // Disp Time Check
        if(disp_time>0){
            disp_time--;
            if(disp_time==0){
                com_clear();
            }
        }
        
        // Over Check
        if(over_count>0){
            over_count++;
            if(over_count>=OVER_TIME){
                rx_count=0;
                rx_data=0;
                over_count=0;
            }
        }
        
        // SW Check
        if(SW==0){
            disp_time=DISP_TIME_INIT;
        }
        
	}

}